﻿Imports System.DirectoryServices

Public Class ADHelper

    ''' <summary>
    ''' Returns all available properties for the object provided. If you provide a person, those will include their name, their email address, and so on.
    ''' </summary>
    ''' <param name="pathToAD"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetAllAvailableProperties(ByVal pathToAD As String) As List(Of String)
        Dim tmpList As New List(Of String)
        Dim de As New DirectoryEntry(pathToAD)
        For Each prop As String In de.Properties.PropertyNames
            tmpList.Add(prop)
        Next
        Return tmpList
    End Function

    ''' <summary>
    ''' Returns all objects from the active directory satisfying the filter you set up
    ''' </summary>
    ''' <param name="pathToAD"></param>
    ''' <param name="propertiesToLoad">tell the method which properties you want to include in the resulting list of persons</param>
    ''' <param name="filterToApply">restrict the objects returned by setting a filter, e.g. (objectCategory=Person) will return persons only</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetUsersFromActiveDirectory(ByVal pathToAD As String, ByVal propertiesToLoad As List(Of String), ByVal filterToApply As String) As List(Of Entity)

        Dim ds As New DirectorySearcher()
        Dim sres As SearchResultCollection
        Dim persons As New List(Of Entity)
        Dim tmpDict As New Dictionary(Of String, String)

        ds = New DirectorySearcher(New DirectoryEntry(pathToAD))
        ds.SearchScope = SearchScope.Subtree
        For Each element As String In propertiesToLoad
            ds.PropertiesToLoad.Add(element)
        Next
        ds.Filter = filterToApply
        sres = ds.FindAll()

        For Each s As SearchResult In sres
            ' all entries are returned as a dictionary
            For Each dictEntry As DictionaryEntry In s.Properties
                ' build up a list of all available properties (the values do hide, that's why we do not simply copy the dict)
                tmpDict.Add(dictEntry.Key.ToString(), GetSingleValue(dictEntry.Value))
            Next
            ' now add the person to the list and clear our helper dictionary
            persons.Add(New Entity(tmpDict))
            tmpDict.Clear()
        Next

        Return persons

    End Function

    ''' <summary>
    ''' Tries to sort the list of persons in alphabetic order by the given property
    ''' </summary>
    ''' <param name="prop"></param>
    ''' <param name="personsToSort"></param>
    ''' <returns>True if sort did work out, false otherwise - mostly because the persons do not share the same property</returns>
    ''' <remarks></remarks>    
    Public Shared Function SortPersons(ByVal prop As String, ByRef personsToSort As List(Of Entity)) As Boolean        ' sort the list here - if the item did exist, that is        
        Dim tmp As String
        ' first check out if all persons do have that property
        Try
            For Each p As Entity In personsToSort
                tmp = p.UserProperties(prop)
            Next
        Catch ex As KeyNotFoundException
            ' fail gracefully
            Return False
            Exit Function
        End Try
        personsToSort.Sort(Function(x, y) x.UserProperties(prop).CompareTo(y.UserProperties(prop)))
        Return True
    End Function

    Private Shared Function GetSingleValue(ByVal values As ResultPropertyValueCollection) As String
        For Each o As Object In values
            Return o.ToString()
        Next
    End Function


End Class
